# Copyright (C) 2009 Ubixum, Inc. 
#
# This library is free software; you can redistribute it and/or
#
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USAimport struct


import struct 

from fx2load import *

openfx2(0x04b4,0x1004)


def do_bulk():
    # send 100 shorts to ep2
    
    buf=struct.pack ( 'H'*100, *[i for i in range(100)] )
    f.ep_bulk( buf, 0x02, 1000)
    
    # read them back out
    buf='\x00'*200  
    f.ep_bulk( buf, 0x86, 1000)
    

    print struct.unpack ( 'H'*100, buf )
    
[do_bulk() for i in range(3)]



#!/usr/bin/env python
#
# Copyright 2009, Ubixum, Inc
#
# This file copied and modified for fx2lib from the GnuRadio project
#
# Copyright 2004,2006 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

import re
import sys, struct
import os, os.path
from optparse import OptionParser


def hex_to_bytes (s):
    if len (s) & 0x1:
        raise ValueError, "Length must be even"
    r = []
    for i in range (0, len(s), 2):
        r.append (int (s[i:i+2], 16))
    return r
    
def msb (x):
    return (x >> 8) & 0xff

def lsb (x):
    return x & 0xff

class ihx_rec (object):
    def __init__ (self, addr, type, data):
        self.addr = addr
        self.type = type
        self.data = data

class ihx_file (object):
    def __init__ (self):
        self.pat = re.compile (r':[0-9A-F]{10,}')
    def read (self, file):
        r = []
        for line in file:
            line = line.strip().upper ()
            if not self.pat.match (line):
                raise ValueError, "Invalid hex record format"
            bytes = hex_to_bytes (line[1:])
            sum = reduce (lambda x, y: x + y, bytes, 0) % 256
            if sum != 0:
                raise ValueError, "Bad hex checksum"
            lenx = bytes[0]
            addr = (bytes[1] << 8) + bytes[2]
            type = bytes[3]
            data = bytes[4:-1]
            if lenx != len (data):
                raise ValueError, "Invalid hex record (bad length)"
            if type != 0:
                break;
            r.append (ihx_rec (addr, type, data))

        return r


def build_eeprom_image (filename, outfile,vid,pid,devid,cb):
    """Build a ``C2 Load'' EEPROM image.

    For details on this format, see section 3.4.3 of
    the EZ-USB FX2 Technical Reference Manual
    """

    image = [
        0xC2,                           # boot from EEPROM
        lsb (vid),
        msb (vid),
        lsb (pid),
        msb (pid),
        lsb (devid),
        msb (devid),
        cb                               # configuration byte
        ]

    # you could just append all the records..
    # but that would most likely cause a lot of
    # extra headers/addrs to be written
    ihx = ihx_file();
    records = ihx.read(open(filename))

    # create image map of all values we're writing data too
    image_map={}
    for r in records:
        addr=r.addr
        c=0
        l=len(r.data)
        while c<l:
            image_map[addr] = r.data[c]
            addr += 1
            c += 1 
    # now create new records based on contiguous image data 
    max_addr = max(image_map.keys())
    records = []
    start_addr = 0

    while start_addr <= max_addr:
        if not image_map.has_key(start_addr):
            start_addr += 1
            continue
        end_addr = start_addr
        # add continguous data up to 10 bits long (0x3ff)
        # is max size, trm 3.4.3
        size=0
        while image_map.has_key(end_addr) and size < 0x3ff:
            end_addr += 1
            size += 1

        l = end_addr - start_addr
        data = []
        for d in range(l):
            data.append(image_map [ start_addr + d ])
        records.append ( ihx_rec ( start_addr, 0, data ) )
        start_addr = end_addr

    
    # 4 byte header that indicates where to load
    # the immediately follow code bytes.

    for r in records:
        image.extend( [
             msb (len (r.data)),
             lsb (len (r.data)),
             msb (r.addr),
             lsb (r.addr)
             ])
        image.extend(r.data)


    # writes 0 to CPUCS reg (brings FX2 out of reset)
    image.extend (  [
        0x80,
        0x01,
        0xe6,
        0x00,
        0x00
        ] )

    buf=struct.pack ( "B"*len(image), *image )
    print "iic Image Size" , len(buf)
    out=open( outfile, 'w') 
    out.write(buf)
    out.close();

if __name__ == '__main__':
    usage = "usage: %prog [options] firmware.ihx outfile"
    parser = OptionParser (usage=usage)
    parser.add_option ( "-v", "--vid", type="int", default=0x04b4,
                        help="Vendor ID for iic c2 image." )
    parser.add_option ( "-p", "--pid", type="int", default=0x0082,
                        help="Product ID for iic c2 image." )
    parser.add_option ( "-d", "--devid", type="int", default=0,
                        help="Device ID for iic c2 image." )
    parser.add_option ( "-c", "--configbyte", type="int", default=0x04,
                        help="Configuration Byte (i2c & disconnect polarity, default 0x04)" )
    (options, args) = parser.parse_args ()
    if len (args) != 2:
        parser.print_help ()
        sys.exit (1)

    ihx_filename = args[0]
    iic_filename = args[1]
    build_eeprom_image ( ihx_filename, iic_filename, options.vid, options.pid, options.devid, options.configbyte )





# Copyright (C) 2009 Ubixum, Inc. 
#
# This library is free software; you can redistribute it and/or
#
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

import sys
from fx2load import *


def get_eeprom(addr,length):
 assert f.isopen()

 prom_val = '';
 while len(prom_val)<length:
     buf='\x00'*1024 # read 1024 bytes max at a time
     transfer_len = length-len(prom_val) > 1024 and 1024 or length-len(prom_val)
     ret=f.do_usb_command ( buf,
         0xc0,
         0xb1,
       addr+len(prom_val),0,transfer_len )
     if (ret>=0):
        prom_val += buf[:ret]
     else: 
        raise Exception("eeprom read didn't work: %d" % ret )
 return prom_val
  
  
def hexchartoint(c):
    return int(c.encode('hex'),16)
  

def fetch_eeprom():
 """
   See TRM 3.4.2, 3.4,3.
   This function dynamically determines how much data to read for c2 eeprom data and downloads
   the eeprom iic file.
 """
 assert f.isopen()
 # fetch 1st 8 bytes
 prom=get_eeprom(0,8)
 if prom[0] == '\xc0':
    return prom # c0 blocks are 8 bytes long
 if prom[0] != '\xc2': raise Exception ( "Envalid eeprom (%s)" % prom[0].encode('hex') )
 # the length of the 1st data block is bytes 8,9 (0 based)
 read_addr=8
 while True:
    size_read = get_eeprom(read_addr,4) # get the data length and start address
    prom += size_read
    read_addr+=4
    # if this is the end 0x80 0x01 0xe6 0x00, then break
    if size_read == '\x80\x01\xe6\x00': break
    # else it is a data block 
    size = (hexchartoint(size_read[0]) << 8) + hexchartoint(size_read[1])
    print "Next eeprom data size %d" % size
    prom += get_eeprom(read_addr,size)
    read_addr+=size
 # one last byte
 prom += get_eeprom(read_addr,1) # should always be 0
 assert prom[-1] == '\x00'
 return prom
 

def set_eeprom(prom):
   assert f.isopen()
   bytes_written=0;
   while bytes_written<len(prom):
        # attemp 1024 at a time
        to_write=len(prom)-bytes_written > 1024 and 1024 or len(prom)-bytes_written
        print "Writing %d Bytes.." % to_write
        ret=f.do_usb_command(prom[bytes_written:bytes_written+to_write], 0x40,0xb1,bytes_written, 0, to_write, 10000)
        if ret>0:
            bytes_written += ret;
        else:
            raise Exception ( "eeprom write didn't work: %d" % ret )
 

if __name__=='__main__':

   openfx2(0x2A19,0x5441) # vid/pid of eeprom firmware 
   print(repr(fetch_eeprom()))




# Copyright (C) 2009 Ubixum, Inc. 
#
# This library is free software; you can redistribute it and/or
#
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

from time import sleep
from fx2 import fx2

f=fx2.fx2()

# use this functions 1st

def openfx2(vid=0x04b4,pid=0x0082,idx=0):
    f.open(vid,pid,idx)

def reset_device(reset):
 print reset and "Put device in reset" or "Set device to run"
 write_ram (0xe600,reset and '\x01' or '\x00', 1)
 
def write_ram(addr,data,length):
 transferred=0
 while(transferred<length):
  this_transfer_size=length-transferred>1024 and 1024 or length-transferred
  buf=data[transferred:]
  ret=f.do_usb_command( buf,
    0x40,
    0xa0,
    addr+transferred, 0,
    this_transfer_size )
  if (ret>0):
   print "wrote %d bytes" % ret
   transferred+=ret
  else:
   print "Error: %d" % ret
   return

def reset_bix(filename):
 """
  Use this function to reset your firmware.  You'll need to reopen the device afterward.
 """
 reset_device(True)
 bix=open(filename).read()
 print "loading bix file of length: %d" % len(bix) 
 write_ram( 0, bix,len(bix) );
 reset_device(False)
 f.close()




from distutils.core import setup, Extension

fx2_mod = Extension (
 '_fx2',
 sources=['cpp/fx2.cpp', 'cpp/fx2.i'],
 swig_opts=['-c++'],
 include_dirs = ['/usr/include/python2.5'],
 libraries = ['usb-1.0']
)


setup(
 name='fx2',
 version='0.2',
 ext_modules=[fx2_mod],
 scripts=['scripts/fx2load'],
 packages=['fx2load','fx2'],
 package_dir={'fx2':'cpp'}
)
